@@ -1,3 +1,4 @@
+from time import sleep
 from shared import PRICE_FOR_NAME
 
 
@@ -9,6 +10,9 @@ def price(_input_consumed, _results_consumed, results_provided):
 
     result = sum([count * unit_price for count, unit_price in zip(counts, unit_prices)])
 
+    # Delay to see the effect of worker caching.
+    # If too fast the watcher loop will not update the results before all calcs are done
+    sleep(0.1)
     results_provided["car_price"] = result
 
 
