@@ -2,4 +2,4 @@ from .utils import get_model
 
 hmodel = get_model()
 hmodel.render()
-hmodel.render(queries=["segments[:].layers[:].outer_temperature"])
+hmodel.render(query=["segments[:].layers[:].outer_temperature"])
